# coding: utf-8

"""
    Co-WIN Protected APIs

    Co-WIN APIs to register for vaccination, to book a vaccination appointment from partner applications. These APIs have restricted access. Please contact Ministry of Health and Family Welfare, Government of India to get access to these APIs. <i>[Updated on 25 Apr 2021]</i>  # noqa: E501

    The version of the OpenAPI document: 2.1.0
    Generated by: https://openapi-generator.tech
"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "cowin-api"
VERSION = "1.0.1"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["pydantic", "httpx", "fastapi", "typing_extensions"]

setup(
    name=NAME,
    version=VERSION,
    description="Co-WIN Protected APIs",
    author_email="",
    url="",
    keywords=["Swagger", "Co-WIN Protected APIs"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "cowin_api": ["py.typed"],
    },
    long_description="""\
    Co-WIN APIs to register for vaccination, to book a vaccination appointment from partner applications. These APIs have restricted access. Please contact Ministry of Health and Family Welfare, Government of India to get access to these APIs. &lt;i&gt;[Updated on 25 Apr 2021]&lt;/i&gt;  # noqa: E501
    """
)
